@@ -1,23 +1,4 @@
-//! Utilities to validate and resolve IRIs following [RFC 3987](https://www.ietf.org/rfc/rfc3987).
-//!
-//! ```
-//! use oxiri::Iri;
-//!
-//! // Parse and validate base IRI
-//! let base_iri = Iri::parse("http://foo.com/bar/baz")?;
-//!
-//! // Validate and resolve relative IRI
-//! let iri = base_iri.resolve("bat#foo")?;
-//! assert_eq!(iri.as_str(), "http://foo.com/bar/bat#foo");
-//!
-//! // Extract IRI components
-//! assert_eq!(iri.scheme(), "http");
-//! assert_eq!(iri.authority(), Some("foo.com"));
-//! assert_eq!(iri.path(), "/bar/bat");
-//! assert_eq!(iri.query(), None);
-//! assert_eq!(iri.fragment(), Some("foo"));
-//! # Result::<(), oxiri::IriParseError>::Ok(())
-//! ```
+#![doc = include_str!("../README.md")]
 #![deny(
     future_incompatible,
     nonstandard_style,
@@ -39,7 +20,7 @@ use std::net::{AddrParseError, Ipv6Addr};
 use std::ops::Deref;
 use std::str::{Chars, FromStr};
 
-/// A [RFC 3987](https://www.ietf.org/rfc/rfc3987) IRI reference.
+/// A [RFC 3987](https://www.ietf.org/rfc/rfc3987.html) IRI reference.
 ///
 /// Instances of this type may be absolute or relative,
 /// unlike [`Iri`].
@@ -68,7 +49,7 @@ pub struct IriRef<T> {
 }
 
 impl<T: Deref<Target = str>> IriRef<T> {
-    /// Parses and validates the IRI-reference following the grammar from [RFC 3987](https://www.ietf.org/rfc/rfc3987).
+    /// Parses and validates the IRI-reference following the grammar from [RFC 3987](https://www.ietf.org/rfc/rfc3987.html).
     ///
     /// This operation keeps internally the `iri` parameter and does not allocate.
     ///
@@ -84,7 +65,7 @@ impl<T: Deref<Target = str>> IriRef<T> {
     }
 
     /// Validates and resolved a relative IRI against the current IRI
-    /// following [RFC 3986](https://www.ietf.org/rfc/rfc3986) relative URI resolution algorithm.
+    /// following [RFC 3986](https://www.ietf.org/rfc/rfc3986.html) relative URI resolution algorithm.
     ///
     /// ```
     /// use oxiri::IriRef;
@@ -104,7 +85,7 @@ impl<T: Deref<Target = str>> IriRef<T> {
     }
 
     /// Validates and resolved a relative IRI against the current IRI
-    /// following [RFC 3986](https://www.ietf.org/rfc/rfc3986) relative URI resolution algorithm.
+    /// following [RFC 3986](https://www.ietf.org/rfc/rfc3986.html) relative URI resolution algorithm.
     ///
     /// It outputs the resolved IRI into `target_buffer` to avoid any memory allocation.
     ///
@@ -471,7 +452,7 @@ impl<'a> From<&'a IriRef<Cow<'a, str>>> for IriRef<&'a str> {
     }
 }
 
-/// A [RFC 3987](https://www.ietf.org/rfc/rfc3987) IRI.
+/// A [RFC 3987](https://www.ietf.org/rfc/rfc3987.html) IRI.
 ///
 /// Instances of this type are guaranteed to be absolute,
 /// unlike [`IriRef`].
@@ -498,7 +479,7 @@ impl<'a> From<&'a IriRef<Cow<'a, str>>> for IriRef<&'a str> {
 pub struct Iri<T>(IriRef<T>);
 
 impl<T: Deref<Target = str>> Iri<T> {
-    /// Parses and validates the IRI following the grammar from [RFC 3987](https://www.ietf.org/rfc/rfc3987).
+    /// Parses and validates the IRI following the grammar from [RFC 3987](https://www.ietf.org/rfc/rfc3987.html).
     ///
     /// This operation keeps internally the `iri` parameter and does not allocate.
     ///
@@ -513,7 +494,7 @@ impl<T: Deref<Target = str>> Iri<T> {
     }
 
     /// Validates and resolved a relative IRI against the current IRI
-    /// following [RFC 3986](https://www.ietf.org/rfc/rfc3986) relative URI resolution algorithm.
+    /// following [RFC 3986](https://www.ietf.org/rfc/rfc3986.html) relative URI resolution algorithm.
     ///
     /// ```
     /// use oxiri::Iri;
@@ -528,7 +509,7 @@ impl<T: Deref<Target = str>> Iri<T> {
     }
 
     /// Validates and resolved a relative IRI against the current IRI
-    /// following [RFC 3986](https://www.ietf.org/rfc/rfc3986) relative URI resolution algorithm.
+    /// following [RFC 3986](https://www.ietf.org/rfc/rfc3986.html) relative URI resolution algorithm.
     ///
     /// It outputs the resolved IRI into `target_buffer` to avoid any memory allocation.
     ///
@@ -1181,7 +1162,11 @@ impl<'a, O: OutputBuffer> IriParser<'a, O> {
                     self.output_positions.scheme_end = base.positions.scheme_end;
                     self.output_positions.authority_end = base.positions.authority_end;
                     self.output_positions.path_end = base.positions.path_end;
-                    self.remove_last_segment_leaving_slash();
+                    self.remove_last_segment();
+                    if self.output.len() > base.positions.scheme_end {
+                        // We have some path or authority, we keep a base '/'
+                        self.output.push('/');
+                    }
                     self.parse_path()
                 }
             }
@@ -1395,17 +1380,6 @@ impl<'a, O: OutputBuffer> IriParser<'a, O> {
             .truncate(last_slash_position + self.output_positions.authority_end)
     }
 
-    fn remove_last_segment_leaving_slash(&mut self) {
-        let last_slash_position =
-            self.output.as_str()[self.output_positions.authority_end..].rfind('/');
-        let truncate_point = self.output_positions.authority_end
-            + match last_slash_position {
-                None => 0,
-                Some(pos) => pos + 1,
-            };
-        self.output.truncate(truncate_point)
-    }
-
     #[inline]
     fn read_url_codepoint_or_echar(&mut self, c: char) -> Result<(), IriParseError> {
         if c == '%' {
@@ -1481,7 +1455,8 @@ fn is_url_code_point(c: char) -> bool {
         | '_'
         | '~'
         | '\u{A0}'..='\u{D7FF}'
-        | '\u{FDF0}'..='\u{FFFD}'
+        | '\u{F900}'..='\u{FDCF}'
+        | '\u{FDF0}'..='\u{FFEF}'
         | '\u{10000}'..='\u{1FFFD}'
         | '\u{20000}'..='\u{2FFFD}'
         | '\u{30000}'..='\u{3FFFD}'
@@ -1500,47 +1475,6 @@ fn is_url_code_point(c: char) -> bool {
 }
 
 fn is_url_query_code_point(c: char) -> bool {
-    matches!(c,
-        'a'..='z'
-        | 'A'..='Z'
-        | '0'..='9'
-        | '!'
-        | '$'
-        | '&'
-        | '\''
-        | '('
-        | ')'
-        | '*'
-        | '+'
-        | ','
-        | '-'
-        | '.'
-        | '/'
-        | ':'
-        | ';'
-        | '='
-        | '?'
-        | '@'
-        | '_'
-        | '~'
-        | '\u{A0}'..='\u{D7FF}'
-        | '\u{E000}'..='\u{FDCF}'
-        | '\u{FDF0}'..='\u{FFFD}'
-        | '\u{10000}'..='\u{1FFFD}'
-        | '\u{20000}'..='\u{2FFFD}'
-        | '\u{30000}'..='\u{3FFFD}'
-        | '\u{40000}'..='\u{4FFFD}'
-        | '\u{50000}'..='\u{5FFFD}'
-        | '\u{60000}'..='\u{6FFFD}'
-        | '\u{70000}'..='\u{7FFFD}'
-        | '\u{80000}'..='\u{8FFFD}'
-        | '\u{90000}'..='\u{9FFFD}'
-        | '\u{A0000}'..='\u{AFFFD}'
-        | '\u{B0000}'..='\u{BFFFD}'
-        | '\u{C0000}'..='\u{CFFFD}'
-        | '\u{D0000}'..='\u{DFFFD}'
-        | '\u{E1000}'..='\u{EFFFD}'
-        | '\u{F0000}'..='\u{FFFFD}'
-        | '\u{100000}'..='\u{10FFFD}'
-    )
+    is_url_code_point(c)
+        || matches!(c, '\u{E000}'..='\u{F8FF}' | '\u{F0000}'..='\u{FFFFD}' | '\u{100000}'..='\u{10FFFD}')
 }
