@@ -30,6 +30,8 @@ fn test_parsing() {
         "http://example.com/foo/bar?q=1&r=2#toto",
         "http://example.com/foo/bar/?q=1&r=2#toto",
         "http://example.com/foo/bar/.././baz",
+        "http://a.example/AZaz\u{00C0}\u{00D6}\u{00D8}\u{00F6}\u{00F8}\u{02FF}\u{0370}\u{037D}\u{037F}\u{1FFF}\u{200C}\u{200D}\u{2070}\u{218F}\u{2C00}\u{2FEF}\u{3001}\u{D7FF}\u{FA0E}\u{FDCF}\u{FDF0}\u{FFEF}\u{10000}\u{EFFFD}",
+        "http://a.example/?AZaz\u{E000}\u{F8FF}\u{F0000}\u{FFFFD}\u{100000}\u{10FFFD}\u{00C0}\u{00D6}\u{00D8}\u{00F6}\u{00F8}\u{02FF}\u{0370}\u{037D}\u{037F}\u{1FFF}\u{200C}\u{200D}\u{2070}\u{218F}\u{2C00}\u{2FEF}\u{3001}\u{D7FF}\u{FA0E}\u{FDCF}\u{FDF0}\u{FFEF}\u{10000}\u{EFFFD}"
     ];
 
     for e in examples.iter() {
@@ -192,6 +194,34 @@ fn test_wrong_relative_parsing() {
         // bad characters
         "//\u{FFFF}",
         "?\u{FFFF}",
+        "/\u{0000}",
+        "?\u{0000}",
+        "#\u{0000}",
+        "/\u{E000}",
+        "/\u{F8FF}",
+        "/\u{F0000}",
+        "/\u{FFFFD}",
+        "/\u{100000}",
+        "/\u{10FFFD}",
+        "?\u{FDEF}",
+        "?\u{FFFF}",
+        "/\u{FDEF}",
+        "/\u{FFFF}",
+        "/u{1FFFF}",
+        "/u{2FFFF}",
+        "/u{3FFFF}",
+        "/u{4FFFF}",
+        "/u{5FFFF}",
+        "/u{6FFFF}",
+        "/u{7FFFF}",
+        "/u{8FFFF}",
+        "/u{9FFFF}",
+        "/u{AFFFF}",
+        "/u{BFFFF}",
+        "/u{CFFFF}",
+        "/u{DFFFF}",
+        "/u{EFFFF}",
+        "/u{FFFFF}",
         // bad host
         "http://[/",
         "http://[::1]a/",
@@ -483,6 +513,7 @@ fn test_resolve_relative_iri() {
         ("/lv2.h", "file:foo", "file:/lv2.h"),
         ("///lv2.h", "file:foo", "file:///lv2.h"),
         ("lv2.h", "file:foo", "file:lv2.h"),
+        ("s", "http://example.com", "http://example.com/s"),
     ];
 
     for (relative, base, output) in examples.iter() {
